==> ump-0.8.3.2-src.tar.bz2: ump-0.8.3.2/src/math2.h <==

/*
 *  Math2
 *  Copyright (c) 2003-2006 by Mattias Hultgren <mattias_hultgren@tele2.se>
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

/*  See math2.cpp for news by version info  */


#ifndef MATH2_H_
#define MATH2_H_

#include "vartypes.h"
#include "picture.h"
#include "vector.h"
#include "integer.h"
#include "real.h"
#include "complex.h"
#include "matrix.h"
#include <math.h>
#include <string>


#define MATH2_H_VERSION "v13b"
#define MATH2_H_DATE    "2003-02 - 2006-04"

namespace math
{


// this function must be called before any thing else is called
void init(void) throw(error_obj);


// this variable desides whether radians or degree should be used by sin, arg...
extern Complex degfix;

/*  priority list
priority
number

 1     []    // Matrix brackets
 2     ()
 3     ""
 4     methods such as set_pixel, get_type...
 5     functions such as sin,cos,sqrt,ipart,time,++,--...
 6     !
 7     ^
 8     * / \ mod
 9     + -
 10    == != < > <= >= ~=
 11    and or xor not
 12    =, +=, -=, *=, /=, \=, ^=

*/


class Variable;
class VariableList;
class CodeLine;
class Picture;
class CodeBlock;
class Array;



typedef void *      void_ptr;     // these typedef's are used to do C++ style castings
typedef Variable *  Variable_ptr;
typedef void (*BuiltinFunctionPtr)(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist);

void add_function(const std::string &name, BuiltinFunctionPtr newfunction,bool needleft,bool needright) throw(error_obj);

// this function returns true if a file was autoloaded
bool autoload_file( const char *name ) throw(error_obj);

// this is the directories that will be checked by autoload_file
extern Vector<std::string> autoload_directories;

enum argument_types{ type_end,
                     type_complex,  type_complex_default,
                     type_floatx,   type_floatx_default,
                     type_int64,    type_int64_default,
                     type_integer,  type_integer_default,
                     type_boolean,  type_boolean_default,
                     type_picture,  type_picture_default,
                     type_matrix,   type_matrix_default,
                     type_string,   type_string_default,
                     type_variable, type_variable_default,
                     type_name,     type_name_default };

void argument_check(const char *function, const Variable *input, ...) throw(error_obj);

void colorrgb_to_colormatrix(const picture_h::colorrgb &color, Matrix *matr) throw(error_obj);
void colormatrix_to_colorrgb(const Matrix &matr, picture_h::colorrgb *color, uint32 index=1)
                             throw(error_obj);
void colormatrix_to_colorrgbs(const Matrix &matr, Vector<picture_h::colorrgb> &colors)
                              throw(error_obj);

void color_list_get_color( const Vector<picture_h::colorrgb> &colors, floatx min, floatx max,
                           floatx value, picture_h::colorrgb *out_color);

// set this pointer to make the intern functions format numbers the way you want
extern Format *intern_format;

extern VariableList global_varlist;


class Picture
{
private:
	floatx left,right,top,bottom;

public:
	picture_h::picture pic;

	picture_h::colorrgb bgcolor;

	Picture() throw(error_obj);
	Picture(const Picture &src) throw(error_obj);

	void operator=(const Picture &src) throw(error_obj);

	bool operator==(const Picture &pict) const;

	void set_size(int32 width,int32 height,floatx newleft,floatx newright,floatx newbottom,floatx newtop) throw(error_obj);

	void set_window(floatx newleft,floatx newright,floatx newbottom,floatx newtop) throw(error_obj);

	inline void clear(void) { pic.clear(bgcolor); }

	void line(floatx x1, floatx y1, floatx x2, floatx y2, bool antialiasing = false,
	          const picture_h::colorrgb &color = picture_h::color::BLACK);

	void point(floatx x,floatx y,const picture_h::colorrgb &color = picture_h::color::BLACK);

	inline floatx get_left(void) const { return left; }
	inline floatx get_right(void) const { return right; }
	inline floatx get_top(void) const { return top; }
	inline floatx get_bottom(void) const { return bottom; }

	floatx get_x_value(int32 xpixel) const;
	floatx get_y_value(int32 ypixel) const;

	int32 get_x_pixel(floatx xvalue) const;
	int32 get_y_pixel(floatx yvalue) const;

	float get_float_x_pixel(floatx xvalue) const;
	float get_float_y_pixel(floatx yvalue) const;


	floatx get_unit_x_pixel(void) const;
	floatx get_unit_y_pixel(void) const;
};



// Variable types
enum VariableType { VariableType_Complex, VariableType_Matrix,  VariableType_Array,
                    VariableType_String,  VariableType_Picture,
                    VariableType_Boolean };

extern const char *VARIABLE_TYPE_NAMES[];



void check_name_against_reserved(const std::string &name) throw(error_obj);

class Variable
{
private:
	VariableType type;

	union{
		bool    boolean;
		Complex *value;
		Matrix  *matr;
		struct{
			CodeLine   *code_line;
			std::string *str;
		};
		Array   *array;
		Picture *pic;
	};

	std::string name;

	void (*change_func)(Variable *ptr);
	void (*delete_func)(Variable *ptr);

	bool inuse;
	bool isconstant;
	bool can_change_type;

public:

	Variable();
	Variable(const Variable &src) throw(error_obj);
	~Variable();

	// if isconstant and can_change_type allows
	// this function copies the value/Matrix/function/string/list of the Variable
	// the name of the Variable will not be touched
	void operator=(const Variable &src) throw(error_obj);

	bool operator==( const Variable &var ) const;
	inline bool operator!=( const Variable &var ) { return !(*this == var); }

	inline bool get_inuse(void) const { return inuse; }
	
	void set_void(void);
	bool is_void(void) const;

	void set_to_constant(void);
	inline bool get_isconstant(void) const { return isconstant; }

	void set_can_change_type(bool boolean) throw(error_obj);
	inline bool get_can_change_type(void) const { return can_change_type; }

	inline void set_to_unused(void) { inuse = false; }


	void set_name(const std::string &newname) throw(error_obj);
	inline void set_name_from(const Variable &src) throw(error_obj) { set_name(src.name); }
	const std::string* get_name(void) const throw(error_obj);

	VariableType get_type(void) const { return type; }

	inline bool has_name(void) const { return !(name.empty()); }

	void signal_changed(void);

	void clear(void) throw(error_obj);

	void set_complex(const Complex &newvalue) throw(error_obj);
	const Complex* get_complex(void) const throw(error_obj);
	Complex* get_complex_rw(void) throw(error_obj);

	void set_matrix(const Matrix *newmatrix) throw(error_obj);
	const Matrix* get_matrix(void) const throw(error_obj);
	Matrix* get_matrix_rw(void) throw(error_obj);


	void set_string(const std::string &newstring) throw(error_obj);
	void set_code_line(const char *new_code_line) throw(error_obj);

	const std::string* get_string(void) const throw(error_obj);
	std::string* get_string_rw(void) throw(error_obj);
	// this will calculate the code_line's value/Matrix/list and return it
	void calc_code_line( Variable *res ) throw(error_obj);


	void set_array(const Array *newarray) throw(error_obj);
	const Array* get_array(void) const throw(error_obj);
	Array* get_array_rw(void) throw(error_obj);

	void set_picture(const Picture *newpic) throw(error_obj);
	const Picture* get_picture(void) const throw(error_obj);
	Picture* get_picture_rw(void) throw(error_obj);

	void set_boolean(bool newbool) throw(error_obj);
	bool get_boolean(void) const throw(error_obj);

	// the change-event does only apply to Variable content not the name or flags like isconstant
	inline void set_change_func(void (*new_func)(Variable *ptr))
	              { change_func = new_func; }

	// this function is called by the destructor
	inline void set_delete_func(void (*new_func)(Variable *ptr))
	              { delete_func = new_func; }

	void append_to_string( std::string &str, const Format &fmt, bool show_name=true ) const
	       throw(error_obj);
};

class Array
{
private:
	uint32 nrofvariables;

	Variable **array;

public:
	Array();
	Array(const Array &src) throw(error_obj);
	~Array();

	void clear(void);

	void set_size( uint32 new_size ) throw(error_obj);

	// this works as set_size except that the content remains as much as possible
	void change_size_to( uint32 new_size ) throw(error_obj);

	inline uint32 get_size(void) const { return nrofvariables; }

	const Variable * get_variable(uint32 nr) const throw(error_obj);

	void set_variable( uint32 nr, const Variable &new_var) throw(error_obj);

	// if var == 0 nothing will happen
	void add_variable(const Variable *var) throw(error_obj);

	// after this call the Array owns the variable
	void add_this_variable( Variable *var ) throw(error_obj);

	void insert( const Variable *var, uint32 at_pos ) throw(error_obj);
	void insert_this( Variable *var, uint32 at_pos ) throw(error_obj);

	// if nr isn't existing nothing happens
	void remove_range( uint32 from_nr, uint32 this_many = 1 );

	void operator=(const Array &src) throw(error_obj);

	bool operator==(const Array &arr) const;
};




class VariableList
{
private:
	struct intern_variablelist
	{
		intern_variablelist *next;

		uint32 idnr;
		uint32 isdeletable;
		Variable *var;

		intern_variablelist();
		~intern_variablelist(); // deletes all 'intern_variablelist'-structs linked to *next
	}*varlist;

public:
	VariableList();
	VariableList(const VariableList &src) throw(error_obj);
	~VariableList();

	void operator=(const VariableList &src) throw(error_obj);

	// returns the idnr for the variable thats called name (if there is more than one with that name the first found is returned)
	uint32 get_id(const char *name) const throw(error_obj);

	// returns the highest idnr in the list
	uint32 get_highest_idnr(void) const;

	// returns a pointer to the variable determind by idnr
	const Variable* get_pointer(uint32 idnr) const throw(error_obj);
	Variable* get_pointer_rw(uint32 idnr) throw(error_obj);

	// creates a new Variable, a pointer to the created Variable is returned...
	Variable * create(const Variable &newvar, bool isconstant, bool isdeletable,
	                  bool can_change_type) throw(error_obj);

	// adds a Variable note that the VariableList will be the owner of Variable newvar after this
	void add_this(Variable *newvar, bool isconstant, bool isdeletable, bool can_change_type) throw(error_obj);

	void set_all_to_unused(void);

	bool get_deletable(uint32 idnr) const throw(error_obj);

	void make_undeletable(uint32 idnr) throw(error_obj);

	// deletes a Variable, if it's deletable
	void remove(uint32 idnr) throw(error_obj);

	// removes all Variables in the list, ignoring the isdeletable-flag
	void clear(void);

	bool is_empty(void) const { return (varlist) ? false : true; }
};




class CodeLine
{
private:
	class CodeLineNode
	{
	public:
		bool               soft_function;
		BuiltinFunctionPtr function;
		std::string        function_name;
		int32              tmpvariable_index; // this is the result variable

		struct{
			int32       tmpvariable_index;
			bool        soft;
			Variable    *ptr;
			std::string name;
		}left, right;

		CodeLineNode() { }
		CodeLineNode(const CodeLineNode &src) throw(error_obj)  {  *this = src;  }
		~CodeLineNode() { }
		void operator=(const CodeLineNode &src) throw(error_obj)
		{
			try
			{
				soft_function = src.soft_function;
				function = src.function;
				function_name = src.function_name;
				tmpvariable_index = src.tmpvariable_index;
				left.tmpvariable_index = src.left.tmpvariable_index;
				left.soft = src.left.soft;
				left.ptr = src.left.ptr;
				left.name = src.left.name;
				right.tmpvariable_index = src.right.tmpvariable_index;
				right.soft = src.right.soft;
				right.ptr = src.right.ptr;
				right.name = src.right.name;
			}
			catch(...)  {  Throw_Couldnt_get_memory();  }
		}
		
	};
	Vector<CodeLineNode> nodes;
	int32                nroftmpvariables;
	std::string          string;
	VariableList         autovarlist;
	VariableList         *private_varlist;

	// this has a void* because I don't want to make struct Symbol visible in this header
	// the void* should be a Vector<Symbol>*
	void create_nodes(void *ptr, int start_at) throw(error_obj);
	void round_bracket_reducing(void *ptr, int start_at) throw(error_obj);
	void matrix_bracket_reducing(void *ptr) throw(error_obj);

	// this function returns the index of the tmp variable which is the matrix...
	int32 parse_matrix_creation( uint32 col, uint32 row ) throw(error_obj);
public:
	CodeLine();
	CodeLine(CodeLine &src) throw(error_obj);
	~CodeLine();

	void operator=(CodeLine &src) throw(error_obj);

	void set_code_line(const char * newstring, bool allow_variable_creation=true,
	                   VariableList *new_private_varlist=0) throw(error_obj);

	void get_code_line( std::string *ret_string ) throw(error_obj);

	// calculates the string that has been set by set_code_line/append_code_line
	void calc( Variable *answer, Vector<Variable> *tmp_variables=0 ) throw(error_obj);

	inline int32 get_nroftmpvariables(void) { return nroftmpvariables; }

	bool is_empty(void) const {  return (nodes.get_size() == 0);  }
};

class CodeBlock
{
private:
	enum CodeBlock_Type { CodeBlock_Type_Void, CodeBlock_Type_Code, CodeBlock_Type_Start,
	                      CodeBlock_Type_Stop };
	enum CodeBlock_SubType { CodeBlock_SubType_None, CodeBlock_SubType_While,
	                         CodeBlock_SubType_If, CodeBlock_SubType_Else,
	                         CodeBlock_SubType_Return,
	                         CodeBlock_SubType_Break, CodeBlock_SubType_Continue };
	std::string filename;

	class CodeBlockLine
	{
	public:
		int               line_number;
		CodeBlock_Type    type;
		CodeBlock_SubType sub_type;

		union{
			CodeLine *code;
			int      pair_with;
		};

		CodeBlockLine();
		CodeBlockLine(const CodeBlockLine &src) throw(error_obj);
		~CodeBlockLine();
		void operator=(const CodeBlockLine &src) throw(error_obj);
	};

	Vector<CodeBlockLine> lines;
	uint32                inuse;
	VariableList          private_varlist;
	int32                 nroftmpvariables;
	
	void add_void( CodeBlock_SubType sub_type ) throw(error_obj);

	void add_line(const char *codeline, int linenr, bool allow_variable_creation,
	              CodeBlock_SubType sub_type = CodeBlock_SubType_None) throw(error_obj);
	void add_curley( CodeBlock_Type type, CodeBlock_SubType sub_type = CodeBlock_SubType_None,
	                 int pos = -1 ) throw(error_obj);

	void set_code_intern(char *str, bool allow_variable_creation ) throw(error_obj);
	bool create_function(char *str, const char *only_allow_identifier) throw(error_obj);

// only None, Break, Continue & Return are used
	void run_block( Vector<Variable> *tmp_variables, CodeBlock_SubType &ret, Variable *ans, int &start,
	                bool expect_curley_ending = false ) const  throw(error_obj);

	void pair_up_curleys(void);
	void add_automatic_curleys(void) throw(error_obj);

public:
	CodeBlock() { inuse = 0; nroftmpvariables = 0; }
	CodeBlock(const CodeBlock &src) throw(error_obj);
	~CodeBlock();

	inline bool get_inuse(void) const { return (inuse != 0); }

	void set_code(const char *newcode) throw(error_obj);
	void set_code_from_file( const std::string &new_filename,
	                         const char *only_allow_identifier = 0 ) throw(error_obj);

	void calc( Variable *answer ) throw(error_obj);

	void clear(void);

	void operator=(const CodeBlock &src) throw(error_obj);

	friend void wrapper_code_block_functions(Variable *res,Variable *left,const Variable *right,
	                                         const std::string &name) throw(error_obj);
	friend void wrapper_check_argument( int i, int i2, const Variable *right ) throw(error_obj);
};


} // namespace math

#endif // MATH2_H_

==> ump-0.8.4-src.tar.bz2: ump-0.8.4/src/math2.h <==

/*
 *  Math2
 *  Copyright (c) 2003-2006 by Mattias Hultgren <mattias_hultgren@tele2.se>
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

/*  See math2.cpp for news by version info  */


#ifndef MATH2_H_
#define MATH2_H_

#include "vartypes.h"
#include "picture.h"
#include "vector.h"
#include "integer.h"
#include "real.h"
#include "complex.h"
#include "matrix.h"
#include <math.h>
#include <string>


#define MATH2_H_VERSION "v14"
#define MATH2_H_DATE    "2003-02 - 2006-05"

namespace math
{


// this function must be called before any thing else is called
void init(void) throw(error_obj);


// this variable desides whether radians or degree should be used by sin, arg...
extern Complex degfix;

/*  priority list
priority
number

 1     []    // Matrix brackets
 2     ()
 3     ""
 4     methods such as set_pixel, get_type...
 5     functions such as sin,cos,sqrt,ipart,time,++,--...
 6     !
 7     ^
 8     * / \ mod
 9     + -
 10    == != < > <= >= ~=
 11    and or xor not
 12    =, +=, -=, *=, /=, \=, ^=

*/


class Variable;
class VariableList;
class CodeLine;
class Picture;
class CodeBlock;
class Array;



typedef void *      void_ptr;     // these typedef's are used to do C++ style castings
typedef Complex *   Complex_ptr;
typedef Variable *  Variable_ptr;
typedef void (*BuiltinFunctionPtr)(Variable *res, Variable *left, const Variable *right, VariableList *private_varlist);

void add_function(const std::string &name, BuiltinFunctionPtr newfunction,bool needleft,bool needright) throw(error_obj);

// this function returns true if a file was autoloaded
bool autoload_file( const char *name ) throw(error_obj);

// this is the directories that will be checked by autoload_file
extern Vector<std::string> autoload_directories;

enum argument_types{ type_end,
                     type_complex,  type_complex_default,
                     type_floatx,   type_floatx_default,
                     type_int64,    type_int64_default,
                     type_integer,  type_integer_default,
                     type_boolean,  type_boolean_default,
                     type_picture,  type_picture_default,
                     type_matrix,   type_matrix_default,
                     type_string,   type_string_default,
                     type_variable, type_variable_default,
                     type_name,     type_name_default };

void argument_check(const char *function, const Variable *input, ...) throw(error_obj);

void colorrgb_to_colormatrix(const picture_h::colorrgb &color, Matrix *matr) throw(error_obj);
void colormatrix_to_colorrgb(const Matrix &matr, picture_h::colorrgb *color, uint32 index=1)
                             throw(error_obj);
void colormatrix_to_colorrgbs(const Matrix &matr, Vector<picture_h::colorrgb> &colors)
                              throw(error_obj);

void color_list_get_color( const Vector<picture_h::colorrgb> &colors, floatx min, floatx max,
                           floatx value, picture_h::colorrgb *out_color);

// set this pointer to make the intern functions format numbers the way you want
extern Format *intern_format;

extern VariableList global_varlist;


class Picture
{
private:
	floatx left,right,top,bottom;

public:
	picture_h::picture pic;

	picture_h::colorrgb bgcolor;

	Picture() throw(error_obj);
	Picture(const Picture &src) throw(error_obj);

	void operator=(const Picture &src) throw(error_obj);

	bool operator==(const Picture &pict) const;

	void set_size(int32 width,int32 height,floatx newleft,floatx newright,floatx newbottom,floatx newtop) throw(error_obj);

	void set_window(floatx newleft,floatx newright,floatx newbottom,floatx newtop) throw(error_obj);

	inline void clear(void) { pic.clear(bgcolor); }

	void line(floatx x1, floatx y1, floatx x2, floatx y2, bool antialiasing = false,
	          const picture_h::colorrgb &color = picture_h::color::BLACK);

	void point(floatx x,floatx y,const picture_h::colorrgb &color = picture_h::color::BLACK);

	inline floatx get_left(void) const { return left; }
	inline floatx get_right(void) const { return right; }
	inline floatx get_top(void) const { return top; }
	inline floatx get_bottom(void) const { return bottom; }

	floatx get_x_value(int32 xpixel) const;
	floatx get_y_value(int32 ypixel) const;

	int32 get_x_pixel(floatx xvalue) const;
	int32 get_y_pixel(floatx yvalue) const;

	float get_float_x_pixel(floatx xvalue) const;
	float get_float_y_pixel(floatx yvalue) const;


	floatx get_unit_x_pixel(void) const;
	floatx get_unit_y_pixel(void) const;
};



// Variable types
enum VariableType { VariableType_Complex, VariableType_Matrix,  VariableType_Array,
                    VariableType_String,  VariableType_Picture,
                    VariableType_Boolean, VariableType_Void };

extern const char *VARIABLE_TYPE_NAMES[];



void check_name_against_reserved(const std::string &name) throw(error_obj);

class Variable
{
private:
	VariableType type;

	union{
		struct{
			bool    boolean;
			char    *complex_buf[ sizeof( Complex ) ];
		};
		Matrix  *matr;
		struct{
			CodeLine   *code_line;
			std::string *str;
		};
		Array   *array;
		Picture *pic;
	};

	std::string name;

	void (*change_func)(Variable *ptr);
	void (*delete_func)(Variable *ptr);

	bool inuse;
	bool isconstant;
	bool can_change_type;

public:

	Variable();
	Variable(const Variable &src) throw(error_obj);
	~Variable();

	// if isconstant and can_change_type allows
	// this function copies the value/Matrix/function/string/list of the Variable
	// the name of the Variable will not be touched
	void operator=(const Variable &src) throw(error_obj);

	bool operator==( const Variable &var ) const;
	inline bool operator!=( const Variable &var ) { return !(*this == var); }

	inline bool get_inuse(void) const { return inuse; }
	
	void set_void(void);
	inline bool is_void(void) const { return ( type == VariableType_Void ); }

	void set_to_constant(void);
	inline bool get_isconstant(void) const { return isconstant; }

	void set_can_change_type(bool boolean) throw(error_obj);
	inline bool get_can_change_type(void) const { return can_change_type; }

	inline void set_to_unused(void) { inuse = false; }


	void set_name(const std::string &newname) throw(error_obj);
	inline void set_name_from(const Variable &src) throw(error_obj) { set_name(src.name); }
	const std::string* get_name(void) const throw(error_obj);

	VariableType get_type(void) const { return type; }

	inline bool has_name(void) const { return !(name.empty()); }

	void signal_changed(void);

	void clear(void) throw(error_obj);

	void set_complex(const Complex &newvalue) throw(error_obj);
	const Complex* get_complex(void) const throw(error_obj);
	Complex* get_complex_rw(void) throw(error_obj);

	void set_matrix(const Matrix *newmatrix) throw(error_obj);
	const Matrix* get_matrix(void) const throw(error_obj);
	Matrix* get_matrix_rw(void) throw(error_obj);


	void set_string(const std::string &newstring) throw(error_obj);
	void set_code_line(const char *new_code_line) throw(error_obj);

	const std::string* get_string(void) const throw(error_obj);
	std::string* get_string_rw(void) throw(error_obj);
	// this will calculate the code_line's value/Matrix/list and return it
	void calc_code_line( Variable *res ) throw(error_obj);


	void set_array(const Array *newarray) throw(error_obj);
	const Array* get_array(void) const throw(error_obj);
	Array* get_array_rw(void) throw(error_obj);

	void set_picture(const Picture *newpic) throw(error_obj);
	const Picture* get_picture(void) const throw(error_obj);
	Picture* get_picture_rw(void) throw(error_obj);

	void set_boolean(bool newbool) throw(error_obj);
	bool get_boolean(void) const throw(error_obj);

	// the change-event does only apply to Variable content not the name or flags like isconstant
	inline void set_change_func(void (*new_func)(Variable *ptr))
	              { change_func = new_func; }

	// this function is called by the destructor
	inline void set_delete_func(void (*new_func)(Variable *ptr))
	              { delete_func = new_func; }

	void append_to_string( std::string &str, const Format &fmt, bool show_name=true ) const
	       throw(error_obj);
};

class Array
{
private:
	uint32 nrofvariables;

	Variable **array;

	// this function increases the size with one, the new pointer is set to zero
	void increase_size(void) throw(error_obj);

public:
	Array();
	Array(const Array &src) throw(error_obj);
	~Array();

	void clear(void);

	void set_size( uint32 new_size ) throw(error_obj);

	inline uint32 get_size(void) const { return nrofvariables; }

	const Variable * get_variable(uint32 nr) const throw(error_obj);

	void set_variable( uint32 nr, const Variable &new_var) throw(error_obj);

	// if var == 0 nothing will happen
	void add_variable(const Variable *var) throw(error_obj);

	// after this call the Array owns the variable
	void add_this_variable( Variable *var ) throw(error_obj);

	void insert( const Variable *var, uint32 at_pos ) throw(error_obj);
	void insert_this( Variable *var, uint32 at_pos ) throw(error_obj);

	// if nr isn't existing nothing happens
	void remove_range( uint32 from_nr, uint32 this_many = 1 );

	void operator=(const Array &src) throw(error_obj);

	bool operator==(const Array &arr) const;
};




class VariableList
{
private:
	struct intern_variablelist
	{
		intern_variablelist *next;

		uint32 idnr;
		uint32 isdeletable;
		Variable *var;

		intern_variablelist();
		~intern_variablelist(); // deletes all 'intern_variablelist'-structs linked to *next
	}*varlist;

public:
	VariableList();
	VariableList(const VariableList &src) throw(error_obj);
	~VariableList();

	void operator=(const VariableList &src) throw(error_obj);

	// returns the idnr for the variable thats called name (if there is more than one with that name the first found is returned)
	uint32 get_id(const char *name) const throw(error_obj);

	// returns the highest idnr in the list
	uint32 get_highest_idnr(void) const;

	// returns a pointer to the variable determind by idnr
	const Variable* get_pointer(uint32 idnr) const throw(error_obj);
	Variable* get_pointer_rw(uint32 idnr) throw(error_obj);

	// creates a new Variable, a pointer to the created Variable is returned...
	Variable * create(const Variable &newvar, bool isconstant, bool isdeletable,
	                  bool can_change_type) throw(error_obj);

	// adds a Variable note that the VariableList will be the owner of Variable newvar after this
	void add_this(Variable *newvar, bool isconstant, bool isdeletable, bool can_change_type) throw(error_obj);

	void set_all_to_unused(void);

	bool get_deletable(uint32 idnr) const throw(error_obj);

	void make_undeletable(uint32 idnr) throw(error_obj);

	// deletes a Variable, if it's deletable
	void remove(uint32 idnr) throw(error_obj);

	// removes all Variables in the list, ignoring the isdeletable-flag
	void clear(void);

	bool is_empty(void) const { return (varlist) ? false : true; }
};




class CodeLine
{
private:
	class CodeLineNode
	{
	public:
		bool               soft_function;
		BuiltinFunctionPtr function;
		std::string        function_name;
		int32              tmpvariable_index; // this is the result variable

		struct{
			int32       tmpvariable_index;
			bool        soft;
			Variable    *ptr;
			std::string name;
		}left, right;

		CodeLineNode() { }
		CodeLineNode(const CodeLineNode &src) throw(error_obj)  {  *this = src;  }
		~CodeLineNode() { }
		void operator=(const CodeLineNode &src) throw(error_obj)
		{
			try
			{
				soft_function = src.soft_function;
				function = src.function;
				function_name = src.function_name;
				tmpvariable_index = src.tmpvariable_index;
				left.tmpvariable_index = src.left.tmpvariable_index;
				left.soft = src.left.soft;
				left.ptr = src.left.ptr;
				left.name = src.left.name;
				right.tmpvariable_index = src.right.tmpvariable_index;
				right.soft = src.right.soft;
				right.ptr = src.right.ptr;
				right.name = src.right.name;
			}
			catch(...)  {  Throw_Couldnt_get_memory();  }
		}
		
	};
	Vector<CodeLineNode> nodes;
	int32                nroftmpvariables;
	std::string          string;
	VariableList         autovarlist;

	// this has a void* because I don't want to make struct Symbol visible in this header
	// the void* should be a Vector<Symbol>*
	void create_nodes(void *ptr, int start_at) throw(error_obj);
	void round_bracket_reducing(void *ptr, int start_at) throw(error_obj);
	void matrix_bracket_reducing(void *ptr) throw(error_obj);

	// this function returns the index of the tmp variable which is the matrix...
	int32 parse_matrix_creation( uint32 col, uint32 row ) throw(error_obj);
public:
	CodeLine();
	CodeLine(CodeLine &src) throw(error_obj);
	~CodeLine();

	void operator=(CodeLine &src) throw(error_obj);

	void set_code_line(const char * newstring, bool allow_variable_creation=true,
	                   VariableList *private_varlist=0) throw(error_obj);

	void get_code_line( std::string *ret_string ) throw(error_obj);

	// calculates the string that has been set by set_code_line/append_code_line
	void calc( Variable *answer, VariableList *private_varlist=0, Vector<Variable> *tmp_variables=0 ) throw(error_obj);

	inline int32 get_nroftmpvariables(void) { return nroftmpvariables; }

	bool is_empty(void) const {  return (nodes.get_size() == 0);  }
};

class CodeBlock
{
private:
	enum CodeBlock_Type { CodeBlock_Type_Void, CodeBlock_Type_Code, CodeBlock_Type_Start,
	                      CodeBlock_Type_Stop };
	enum CodeBlock_SubType { CodeBlock_SubType_None, CodeBlock_SubType_While,
	                         CodeBlock_SubType_If, CodeBlock_SubType_Else,
	                         CodeBlock_SubType_Return,
	                         CodeBlock_SubType_Break, CodeBlock_SubType_Continue };
	std::string filename;

	class CodeBlockLine
	{
	public:
		int               line_number;
		CodeBlock_Type    type;
		CodeBlock_SubType sub_type;

		union{
			CodeLine *code;
			int      pair_with;
		};

		CodeBlockLine();
		CodeBlockLine(const CodeBlockLine &src) throw(error_obj);
		~CodeBlockLine();
		void operator=(const CodeBlockLine &src) throw(error_obj);
	};

	Vector<CodeBlockLine> lines;
	uint32                inuse;
	VariableList          private_varlist;
	int32                 nroftmpvariables;
	
	void add_void( CodeBlock_SubType sub_type ) throw(error_obj);

	void add_line(const char *codeline, int linenr, bool allow_variable_creation,
	              CodeBlock_SubType sub_type = CodeBlock_SubType_None) throw(error_obj);
	void add_curley( CodeBlock_Type type, CodeBlock_SubType sub_type = CodeBlock_SubType_None,
	                 int pos = -1 ) throw(error_obj);

	void set_code_intern(char *str, bool allow_variable_creation ) throw(error_obj);
	bool create_function(char *str, const char *only_allow_identifier) throw(error_obj);

// only None, Break, Continue & Return are used
	void run_block( Vector<Variable> *tmp_variables, VariableList *this_private_varlist, CodeBlock_SubType &ret,
	                Variable *ans, int &start, bool expect_curley_ending = false ) const  throw(error_obj);

	void pair_up_curleys(void);
	void add_automatic_curleys(void) throw(error_obj);

public:
	CodeBlock() { inuse = 0; nroftmpvariables = 0; }
	CodeBlock(const CodeBlock &src) throw(error_obj);
	~CodeBlock();

	inline bool get_inuse(void) const { return (inuse != 0); }

	void set_code(const char *newcode) throw(error_obj);
	void set_code_from_file( const std::string &new_filename,
	                         const char *only_allow_identifier = 0 ) throw(error_obj);

	void calc( Variable *answer ) throw(error_obj);

	void clear(void);

	void operator=(const CodeBlock &src) throw(error_obj);

	friend void wrapper_code_block_functions(Variable *res,Variable *left,const Variable *right,
	                                         const std::string &name) throw(error_obj);
	friend void wrapper_check_argument( int i, int i2, const Variable *right ) throw(error_obj);
};


} // namespace math

#endif // MATH2_H_
